from .types import *
import json
from . import deserializers
from . import serializers

__version__ = "0.2.0"


def loads(s: AnyStr) -> GeoJSON:
    return deserializers.geojson(json.loads(s))


def load(f: IO[str]) -> GeoJSON:
    return deserializers.geojson(json.load(f))


def dump(g: GeoJSON, fp: IO[str]) -> None:
    o = serializers.geojson(g)
    return json.dump(o, fp)


def dumps(g: GeoJSON) -> str:
    o = serializers.geojson(g)
    return json.dumps(o)


def loads_feature_collection(s: AnyStr) -> FeatureCollection:
    fc = loads(s)
    if isinstance(fc, FeatureCollection):
        return fc
    raise TypeError("Tried to load an object that was not a FeatureCollection")


def load_feature_collection(f: IO[str]) -> FeatureCollection:
    fc = load(f)
    if isinstance(fc, FeatureCollection):
        return fc
    raise TypeError("Tried to load an object that was not a FeatureCollection")



from typing import Any, List
from .types import (
    Point,
    LineString,
    Coordinates,
    GeoJSONEncodeError,
    GeoJSON,
    Feature,
    GeometryObject,
    BoundingBox,
    Polygon,
    FeatureCollection,
    GeometryCollection,
    MultiPoint,
    MultiLineString,
    MultiPolygon,
)


def boundingbox(o: BoundingBox) -> Any:
    return list(o)


def coordinates1(o: Coordinates) -> Any:
    return list(o)


def coordinates2(o: List[Coordinates]) -> Any:
    return [coordinates1(c) for c in o]


def coordinates3(o: List[List[Coordinates]]) -> Any:
    return [coordinates2(c) for c in o]


def coordinates4(o: List[List[List[Coordinates]]]) -> Any:
    return [coordinates3(c) for c in o]


def feature(o: Feature) -> Any:
    r: Any = {"type": o.type, **o.extra_attributes}
    if o.id:
        r["id"] = o.id
    if o.geometry:
        r["geometry"] = geometry(o.geometry)
    if o.properties is not {}:
        r["properties"] = o.properties
    if o.bbox:
        r["bbox"] = boundingbox(o.bbox)
    return r


def featurecollection(o: FeatureCollection) -> Any:
    r = {
        "type": o.type,
        "features": [feature(f) for f in o.features],
        **o.extra_attributes,
    }
    if o.bbox:
        r["bbox"] = boundingbox(o.bbox)
    return r


def geometry(o: GeometryObject) -> Any:
    if isinstance(o, Point):
        r = {"type": o.type, "coordinates": coordinates1(o.coordinates)}
        if o.bbox:
            r["bbox"] = boundingbox(o.bbox)
        return r
    if isinstance(o, MultiPoint) or isinstance(o, LineString):
        r = {"type": o.type, "coordinates": coordinates2(o.coordinates)}
        if o.bbox:
            r["bbox"] = boundingbox(o.bbox)
        return r
    if isinstance(o, MultiLineString) or isinstance(o, Polygon):
        r = {"type": o.type, "coordinates": coordinates3(o.coordinates)}
        if o.bbox:
            r["bbox"] = boundingbox(o.bbox)
        return r
    if isinstance(o, MultiPolygon):
        r = {"type": o.type, "coordinates": coordinates4(o.coordinates)}
        if o.bbox:
            r["bbox"] = boundingbox(o.bbox)
        return r
    if isinstance(o, GeometryCollection):
        r = {"type": o.type, "geometries": [geometry(g) for g in o.geometries]}
        if o.bbox:
            r["bbox"] = boundingbox(o.bbox)
        return r

    raise GeoJSONEncodeError("Unknown geometry object %s" % (o,))


def geojson(o: GeoJSON) -> Any:
    if (
        isinstance(o, Point)
        or isinstance(o, MultiPoint)
        or isinstance(o, LineString)
        or isinstance(o, MultiLineString)
        or isinstance(o, Polygon)
        or isinstance(o, MultiPolygon)
        or isinstance(o, GeometryCollection)
    ):
        return geometry(o)
    if isinstance(o, Feature):
        return feature(o)
    if isinstance(o, FeatureCollection):
        return featurecollection(o)

    raise GeoJSONEncodeError("Unknown GeoJSON object %s" % (o,))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pygeojson']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pygeojson',
    'version': '0.2.0',
    'description': 'GeoJSON library for Python with types',
    'long_description': '# pygeojson 🗺\n\nGeoJSON library for Python with [types](https://docs.python.org/3/library/typing.html).\n\nThe world doesn\'t need another\n[GeoJSON library for python](https://github.com/jazzband/geojson), but it needs\na better one. The goal of this library is to provide a simple and typed data\nmodel for GeoJSON such that y\'all can get static code check and editor support\nwith [mypy](http://mypy-lang.org/). And of course, data should be data, so the\ndata model is not polluted with inheritance, custom methods, utilities,\nextension or other "conveniences". Lastly, your data will be immutable!\n\n`pygeojson` is built on top of\n[`dataclasses`](https://docs.python.org/3/library/dataclasses.html) to provide\ntypes, immutability and default value support. Models does not inherit from one\nbut instead we use\n[`Union` types](https://docs.python.org/3/library/typing.html#typing.Union)\nwhere applicable.\n\nIn addition, `pygeojson` comes with serialization and deserialization support\nvia the `dump`, `dumps`, `load` and `loads` functions.\n\n## Installation\n\n`pygeojson` requires Python 3.7 and up. Install it via `pipenv`:\n\n```sh\npipenv install pygeojson\n```\n\nor `pip`:\n\n```sh\npip install pygeojson\n```\n\n## Usage\n\nTBD\n\n## Data model\n\nTBA\n',
    'author': 'Håkon Åmdal',
    'author_email': 'hakon@aamdal.com',
    'maintainer': 'Håkon Åmdal',
    'maintainer_email': 'hakon@aamdal.com',
    'url': 'https://github.com/hawkaa/pygeojson',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
